import { ParallelStructure } from "./parallel_structure";

// Classes générales sur les structures
import { RectangularStructureParams } from "./rectangular_structure_params";
import { Structure } from "./structure";
import { LoiDebit } from "./structure_props";

// Equations de débit
import { StructureGateCem88d, StructureWeirCem88d } from "./structure_cem88d";
import { StructureGateCem88v, StructureWeirCem88v } from "./structure_cem88v";
import { StructureCunge80 } from "./structure_cunge80";
import { StructureKivi, StructureKiviParams } from "./structure_kivi";
import { StructureOrificeSubmerged, StructureOrificeSubmergedParams } from "./structure_orifice_submerged";
import { StructureRectangularOrificeFree } from "./structure_rectangular_orifice_free";
import { StructureRectangularOrificeSubmerged } from "./structure_rectangular_orifice_submerged";
// tslint:disable-next-line:max-line-length
import { StructureTriangularTruncWeirFree, TriangularTruncStructureParams } from "./structure_triangular_trunc_weir";
import { StructureTriangularWeir, TriangularStructureParams } from "./structure_triangular_weir";
import { StructureWeirFree } from "./structure_weir_free";
import { StructureWeirSubmergedLarinier } from "./structure_weir_submerged_larinier";

export function CreateStructure(loiDebit: LoiDebit, parentNub?: ParallelStructure,
                                dbg: boolean = false
): Structure {
    let ret: Structure;
    const oCd: {[s: string]: number} = {SeuilR: 0.4, VanneR: 0.6, SeuilT: 1.36};
    const rectStructPrms: RectangularStructureParams = new RectangularStructureParams(
        0,  // Q
        100,        // ZDV
        102,        // Z1
        101.5,      // Z2
        2,          // L
        oCd.SeuilR        // Cd pour un seuil rectangulaire
        // W = Infinity par défaut pour un seuil
    );

    // Control of validity of structure type and definition of parameters
    switch (loiDebit) {
        case LoiDebit.GateCem88d:
        case LoiDebit.GateCem88v:
        case LoiDebit.Cunge80:
        case LoiDebit.RectangularOrificeFree:
        case LoiDebit.RectangularOrificeSubmerged:
            rectStructPrms.W.singleValue = 0.5;
            rectStructPrms.Cd.v = oCd.VanneR; // Cd pour une vanne rectangulaire
    }

    // Instanciation of the equation
    switch (loiDebit) {
        case LoiDebit.WeirCem88d:
            rectStructPrms.Cd.v = oCd.SeuilR; // Cd Cem88d en seuil
            ret =  new StructureWeirCem88d(rectStructPrms, dbg);
            break;

        case LoiDebit.WeirCem88v:
            rectStructPrms.Cd.v = oCd.VanneR; // Cd Cem88d en seuil
            ret =  new StructureWeirCem88v(rectStructPrms, dbg);
            break;

        case LoiDebit.GateCem88d:
            rectStructPrms.Cd.v = oCd.SeuilR; // Cd Cem88d en vanne
            ret =  new StructureGateCem88d(rectStructPrms, dbg);
            break;

        case LoiDebit.GateCem88v:
            rectStructPrms.Cd.v = oCd.VanneR; // Cd Cem88d en vanne
            ret =  new StructureGateCem88v(rectStructPrms, dbg);
            break;

        case LoiDebit.Cunge80:
            rectStructPrms.Cd.v = oCd.VanneR; // Cd Cunge80 qu'on soit en seuil ou vanne
            ret =  new StructureCunge80(rectStructPrms, dbg);
            break;

        case LoiDebit.RectangularOrificeFree:
            ret =  new StructureRectangularOrificeFree(rectStructPrms, dbg);
            break;

        case LoiDebit.RectangularOrificeSubmerged:
            ret =  new StructureRectangularOrificeSubmerged(rectStructPrms, dbg);
            break;

        case LoiDebit.WeirFree:
            ret =  new StructureWeirFree(rectStructPrms, dbg);
            break;

        case LoiDebit.WeirSubmergedLarinier:
        case LoiDebit.CloisonsWeirSubmergedLarinier:
            ret =  new StructureWeirSubmergedLarinier(rectStructPrms, dbg);
            break;

        case LoiDebit.KIVI:
        case LoiDebit.CloisonsKIVI:
            const structKiviPrm: StructureKiviParams = new StructureKiviParams(
                8.516,      // Q
                101.5,        // ZDV
                103,        // Z1
                102,        // Z2
                2,          // L
                oCd.SeuilR, // alpha
                0.001,      // béta
                100);       // ZRAM : cote Radier Amont
            if (loiDebit === LoiDebit.CloisonsKIVI) {
                structKiviPrm.ZDV.visible = false;
                structKiviPrm.h1.visible = true;
                structKiviPrm.h1.singleValue = 1.5;
            }
            ret =  new StructureKivi(structKiviPrm, dbg);
            break;

        case LoiDebit.TriangularWeirFree:
        case LoiDebit.CloisonsTriangularWeirFree:
            const structTriangPrms: TriangularStructureParams = new TriangularStructureParams(
                0,          // Q
                100,        // ZDV
                102,        // Z1
                100,        // Z2
                45,         // Alpha2
                oCd.SeuilT  // Cd pour un seuil triangulaire
                // W = Infinity par défaut pour un seuil
            );
            ret =  new StructureTriangularWeir(structTriangPrms, dbg);
            break;

        case LoiDebit.TriangularTruncWeirFree:
        case LoiDebit.CloisonsTriangularTruncWeirFree:
            const structTriTruncPrms: TriangularTruncStructureParams = new TriangularTruncStructureParams(
                0,          // Q
                100.1,        // ZDV
                102,        // Z1
                100.1,        // Z2
                0.9,        // BT
                101,        // ZT
                oCd.SeuilT  // Cd pour un seuil triangulaire
                // W = Infinity par défaut pour un seuil
            );
            ret =  new StructureTriangularTruncWeirFree(structTriTruncPrms, dbg);
            break;

        case LoiDebit.OrificeSubmerged:
            ret = new StructureOrificeSubmerged(
                new StructureOrificeSubmergedParams(
                    0,      // Q
                    102,    // Z1
                    101.5,  // Z2
                    0.7,    // Cd
                    0.1     // S
                ),
                dbg
            );
            break;

        default:
            throw new Error(`type de LoiDebit ${LoiDebit[loiDebit]} non pris en charge`);
    }

    // set reference to parent
    if (parentNub) {
        ret.parent = parentNub;
    }

    return ret;
}
